<?php 	 

/**
 * PHP Telegram Bot class
 *
 * @package Telegram.class.php
 * @author BotHub
 * @copyright Copyright (c) 2018, BotHub
 * @license https://vlmi.su/license.txt
 * @link https://vlmi.su/
 * @version 0.0.1 Beta
 */

class Telegram {
	private const API_BOT             = 'https://api.telegram.org/bot';
	private const API_FILE            = 'https://api.telegram.org/file/bot';
	private const CHAT_ID             = 'chat_id';
	private const MESSAGE_ID          = 'message_id';
	private const FROM_CHAT_ID        = 'from_chat_id';
	private const PHOTO               = 'photo';
	private const DOCUMENT            = 'document';
	private const TEXT                = 'text';
	private const CAPTION             = 'caption';
	private const REPLY_MARKUP        = 'reply_markup';
	private const PARSE_MODE          = 'parse_mode';
	private const PARSE_MODE_DATA     = 'HTML';
	private const REPLY_TO_MESSAGE_ID = 'reply_to_message_id';
	private const CALLBACK_QUERY_ID   = 'callback_query_id';


	public function __construct(string $token, string $telegramJson = null) {
		$this->token    = $token;
		$this->telegram = json_decode(!empty($telegramJson) ? $telegramJson : file_get_contents('php://input'), true);
	}

	private function checking($data) {
  	$data = trim($data);
		$data = htmlspecialchars($data);

		return $data;
  }

  private function getEntities(array $entities, string $text) {
		$arrEntities = [];
		foreach($entities as $valueEntities) {
			$arrEntities []= [
				'data' => mb_substr($text, $valueEntities['offset'], $valueEntities['length']),
				'type' => $valueEntities['type']
			];
		}
		$arrEntities = array_map("unserialize", array_unique(array_map("serialize", $arrEntities)));
		$arrEntities = array_values($arrEntities);

		return $arrEntities ?? false;
	}

	public function createJson(array $array, string $path) {
		$array  = preg_replace('/^(  +?)\\1(?=[^ ])/m', '$1', json_encode($array, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
		$result = file_put_contents($path, $array);

		return !empty($result) ? true : false;
	}

	public function pathTemplate(string $path) {
  	if (file_exists($path)) {
  		$this->TPLTemplate = file_get_contents($path);
  	}
  }

  public function tagTemplate(string $tag, string $value) {
  	$this->tagsTemplate[$tag] = $value;
  }

  public function showTemplate(string $tag) {
  	preg_match("#\<!-- BEGIN:$tag\ -->(.+?)\<!-- END:$tag\ -->#is", $this->TPLTemplate, $matches);
  	$content = $matches[1];
  	if (!empty($this->tagsTemplate)) {
  		foreach($this->tagsTemplate as $key => $value) {
  			$content = str_replace($key, $value, $content);
  		}
  	}

  	return $content;
  }

  public function pathKeyboard(string $path) {
  	if (file_exists($path)) {
  		$this->jsonKeyboard = json_decode(file_get_contents($path), true);
  	}
  }

  public function setKeyboard(string $key) {
  	if (!empty($this->jsonKeyboard)) {
  		return $this->jsonKeyboard[$key];
  	}
  }

  public function inlineKeyboard(array $buttons) {
  	return ['inline_keyboard' => $buttons];
  }

  public function Keyboard(array $buttons) {
  	return ['keyboard' => $buttons, 'resize_keyboard' => true];
  }

  public function removeKeyboard() {
  	return ['remove_keyboard' => true];
  }

	public function response() {
		return $this->telegram;
	}

	public function typeMessage() {
		if (!empty($this->telegram['callback_query'])) {
			if ($this->telegram['callback_query']['message']['chat']['type'] == 'group') return 'callback_query_group';
			return 'callback_query';
		}

		if (!empty($this->telegram['channel_post'])) {
			return 'channel_post';
		}

		if (!empty($this->telegram['edited_channel_post'])) {
			return 'edited_channel_post';
		}

		if (!empty($this->telegram['inline_query'])) {
			return 'inline_query';
		}

		if (!empty($this->telegram['edited_message'])) {
			if ($this->telegram['edited_message']['chat']['type'] == 'group') {
				if (!empty($this->telegram['edited_message']['reply_to_message'])) {
					if (!empty($this->telegram['edited_message']['entities'])) return 'edited_reply_entities_group';
					return 'edited_reply_group';
				}
				if (!empty($this->telegram['edited_message']['entities'])) return 'edited_entities_group';
				if (!empty($this->telegram['edited_message']['document'])) return 'edited_document_group';
				return 'edited_group';
			}
			if ($this->telegram['edited_message']['chat']['type'] == 'supergroup') {
				if (!empty($this->telegram['edited_message']['reply_to_message'])) {
					if (!empty($this->telegram['edited_message']['entities'])) return 'edited_reply_entities_supergroup';
					return 'edited_reply_supergroup';
				}
				if (!empty($this->telegram['edited_message']['entities'])) return 'edited_entities_supergroup';
				if (!empty($this->telegram['edited_message']['document'])) return 'edited_document_supergroup';
				return 'supergroup';
			}
			if (!empty($this->telegram['edited_message']['reply_to_message'])) return 'edited_reply_message';
			if (!empty($this->telegram['edited_message']['entities'])) return 'edited_entities_message';
			if (!empty($this->telegram['edited_message']['document'])) return 'edited_document_message';
			return 'edited_message';
		}

		if (!empty($this->telegram['message'])) {
			if ($this->telegram['message']['chat']['type'] == 'group') {
				if (!empty($this->telegram['message']['reply_to_message'])) {
					if (!empty($this->telegram['message']['entities'])) return 'reply_entities_group';
					return 'reply_group';
				}
				if (!empty($this->telegram['message']['entities'])) return 'entities_group';
				if (!empty($this->telegram['message']['document'])) return 'document_group';
				return 'group';
			}
			if ($this->telegram['message']['chat']['type'] == 'supergroup') {
				if (!empty($this->telegram['message']['reply_to_message'])) {
					if (!empty($this->telegram['message']['entities'])) return 'reply_entities_supergroup';
					return 'reply_supergroup';
				}
				if (!empty($this->telegram['message']['entities'])) return 'entities_supergroup';
				if (!empty($this->telegram['message']['document'])) return 'document_supergroup';
				return 'supergroup';
			}
			if (!empty($this->telegram['message']['reply_to_message'])) return 'reply_message';
			if (!empty($this->telegram['message']['entities'])) return 'entities_message';
			if (!empty($this->telegram['message']['document'])) return 'document_message';
			return 'message';
		}
	}

	private function message() {
		if (!empty($this->telegram['callback_query']))      return $this->telegram['callback_query'];
		if (!empty($this->telegram['channel_post']))        return $this->telegram['channel_post'];
		if (!empty($this->telegram['edited_channel_post'])) return $this->telegram['edited_channel_post'];
		if (!empty($this->telegram['inline_query']))        return $this->telegram['inline_query'];
		if (!empty($this->telegram['edited_message']))      return $this->telegram['edited_message'];

		return $this->telegram['message'];
	}

	public function chatID() {
		return $this->message()['chat']['id'] ?? $this->message()['message']['chat']['id'] ?? false;
	}

	public function replyChatID() {
		return $this->message()['reply_to_message']['chat']['id'] ?? false;
	}

	public function messageID() {
		return $this->message()['message_id'] ?? $this->message()['message']['message_id'] ?? false;
	}

	public function replyMessageID() {
		return $this->message()['reply_to_message']['message_id'] ?? false;
	}

	public function callbackID() {
		return $this->message()['id'] ?? false;
	}

	public function userID() {
		return $this->message()['from']['id'] ?? false;
	}

	public function replyUserID() {
		return $this->message()['reply_to_message']['from']['id'] ?? false;
	}

	public function firstName() {
		return $this->checking($this->message()['from']['first_name'] ?? false);
	}

	public function replyFirstName() {
		return $this->checking($this->message()['reply_to_message']['from']['first_name'] ?? false);
	}

	public function lastName() {
		return $this->checking($this->message()['from']['last_name'] ?? false);
	}

	public function replyLastName() {
		return $this->checking($this->message()['reply_to_message']['from']['last_name'] ?? false);
	}

	public function username() {
		return $this->checking($this->message()['from']['username'] ?? false);
	}

	public function replyUsername() {
		return $this->checking($this->message()['reply_to_message']['from']['username'] ?? false);
	}

	public function chatUsername() {
		return $this->checking($this->message()['chat']['username'] ?? false);
	}

	public function replyChatUsername() {
		return $this->checking($this->message()['reply_to_message']['chat']['username'] ?? false);
	}

	public function languageCode() {
		return $this->message()['from']['language_code'] ?? false;
	}

	public function replyLanguageCode() {
		return $this->message()['reply_to_message']['from']['language_code'] ?? false;
	}

	public function type() {
		return $this->message()['chat']['type'] ?? $this->message()['message']['chat']['type'] ?? false;
	}

	public function replyType() {
		return $this->message()['reply_to_message']['chat']['type'] ?? false;
	}

	public function title() {
		return $this->message()['chat']['title'] ?? $this->message()['message']['chat']['title'] ?? false;
	}

	public function replyTitle() {
		return $this->message()['reply_to_message']['chat']['title'] ?? false;
	}

	public function date() {
		return $this->message()['date'] ?? $this->message()['message']['date'] ?? false;
	}

	public function editDate() {
		return $this->message()['edit_date'] ?? $this->message()['message']['edit_date'] ?? false;
	}

	public function replyDate() {
		return $this->message()['reply_to_message']['date'] ?? false;
	}

	public function text() {
		return $this->checking($this->message()['text'] ?? false);
	}

	public function replyText() {
		return $this->checking($this->message()['reply_to_message']['text'] ?? false);
	}

	public function callbackData() {
		return $this->checking($this->message()['data'] ?? false);
	}

	public function document() {
		return $this->message()['document'] ?? false;
	}

	public function replyDocument() {
		return $this->message()['reply_to_message']['document'] ?? false;
	}

	public function docFileName() {
		return $this->checking($this->message()['document']['file_name'] ?? false);
	}

	public function replyDocFileName() {
		return $this->checking($this->message()['reply_to_message']['document']['file_name'] ?? false);
	}

	public function docMimeType() {
		return $this->message()['document']['mime_type'] ?? false;
	}

	public function replyDocMimeType() {
		return $this->message()['reply_to_message']['document']['mime_type'] ?? false;
	}

	public function docFileID() {
		return $this->message()['document']['file_id'] ?? false;
	}

	public function replyDocFileID() {
		return $this->message()['reply_to_message']['document']['file_id'] ?? false;
	}

	public function docFileSize() {
		return $this->message()['document']['file_size'] ?? false;
	}

	public function replyDocFileSize() {
		return $this->message()['reply_to_message']['document']['file_size'] ?? false;
	}

	public function entities() {
		return $this->getEntities($this->message()['entities'], $this->text()) ?? false;
	}

	public function replyEntities() {
		return $this->getEntities($this->message()['reply_to_message']['entities'], $this->replyText()) ?? false;
	}

	private function request(string $method, array $parameters, bool $isFile = false) {
		$parameters['method'] = $method;
		if (empty($isFile)) {
			$parameters = json_encode($parameters);
			$header     = array("Content-Type: application/json");
		} else {
			$parameters = $parameters;
			$header     = array("Content-Type: multipart/form-data");
		}
		$ch = curl_init(self::API_BOT . $this->token . '/');
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
		curl_setopt($ch, CURLOPT_TIMEOUT, 60);
		curl_setopt($ch, CURLOPT_POST, true);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $parameters);
		curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
		$response = json_decode(curl_exec($ch), true);
		curl_close($ch);

		return !empty($response['ok']) ? $response['result'] : $response;
	}

	public function setWebhook(string $url = null) {
		$parameters['url'] = $url ? $url : '';

		return $this->request('setWebhook', $parameters);
	}

	public function sendMessage(int $chatID, string $text, array $keyboard = null) {
		$parameters = [
			self::CHAT_ID      => $chatID,
			self::TEXT         => $text,
			self::REPLY_MARKUP => $keyboard ? $keyboard : '',
			self::PARSE_MODE   => self::PARSE_MODE_DATA
		];

		return $this->request('sendMessage', $parameters);
	}

	public function editMessage(int $chatID, int $messageID, string $text, array $keyboard = null) {
		$parameters = [
			self::CHAT_ID      => $chatID,
			self::MESSAGE_ID   => $messageID,
			self::TEXT         => $text,
			self::REPLY_MARKUP => $keyboard ? $keyboard : '',
			self::PARSE_MODE   => self::PARSE_MODE_DATA
		];

		return $this->request('editMessagetext', $parameters);
	}

	public function deleteMessage(int $chatID, int $messageID) {
		$parameters = [
			self::CHAT_ID      => $chatID,
			self::MESSAGE_ID   => $messageID
		];

		return $this->request('deleteMessage', $parameters);
	}

	public function sendAnswer(int $callbackID, string $text) {
		$parameters = [
			self::CALLBACK_QUERY_ID => $callbackID,
			self::TEXT              => $text,
		];

		return $this->request('answercallbackQuery', $parameters);
	}

	public function replyToMessage(int $chatID, int $messageID, string $text, array $keyboard = null) {
		$parameters = [
			self::CHAT_ID             => $chatID,
			self::TEXT                => $text,
			self::REPLY_MARKUP        => $keyboard ? $keyboard : '',
			self::PARSE_MODE          => self::PARSE_MODE_DATA,
			self::REPLY_TO_MESSAGE_ID => $messageID
		];

		return $this->request('sendMessage', $parameters);
	}

	public function forwardMessage(int $chatID, int $fromChatID, int $messageID) {
		$parameters = [
			self::CHAT_ID      => $chatID,
			self::FROM_CHAT_ID => $fromChatID,
			self::MESSAGE_ID   => $messageID
		];

		return $this->request('forwardMessage', $parameters);
	}

	public function sendPhoto(int $chatID, string $photoID, string $text = null, array $keyboard = null) {
		$parameters = [
			self::CHAT_ID      => $chatID,
			self::PHOTO        => $photoID,
			self::CAPTION      => $text ? $text : '',
			self::REPLY_MARKUP => $keyboard ? $keyboard : '',
			self::PARSE_MODE   => self::PARSE_MODE_DATA
		];

		return $this->request('sendPhoto', $parameters);
	}

	public function sendPhotoFile(int $chatID, string $path, string $text = null, array $keyboard = null) {
		$parameters = [
			self::CHAT_ID      => $chatID,
			self::PHOTO        => new CURLFile(realpath($path)),
			self::CAPTION      => $text ? $text : '',
			self::REPLY_MARKUP => $keyboard ? $keyboard : '',
			self::PARSE_MODE   => self::PARSE_MODE_DATA
		];

		return $this->request('sendPhoto', $parameters, true);
	}

	public function sendDocumentFile(int $chatID, string $path, string $text = null, array $keyboard = null) {
		$parameters = [
			self::CHAT_ID      => $chatID,
			self::DOCUMENT     => new CURLFile(realpath($path)),
			self::CAPTION      => $text ? $text : '',
			self::REPLY_MARKUP => $keyboard ? $keyboard : '',
			self::PARSE_MODE   => self::PARSE_MODE_DATA
		];

		return $this->request('sendDocument', $parameters, true);
	}

	public function download(string $fileID, string $path) {
		$parameters['file_id'] = $fileID;
		$info = $this->request('getFile', $parameters);
		if (!empty($info)) {
			$fp = fopen($path, 'w');
			$ch = curl_init(self::API_FILE . $this->token . '/' . $info['file_path']);
			curl_setopt($ch, CURLOPT_FILE, $fp);
			curl_exec($ch);
			curl_close($ch);
			fclose($fp);

			return true;
		}
		return false;
	}
} 

?>